import { getFileUrl } from '@root/service/oss';
import { ScrollView, View, Image, Text } from '@tarojs/components';
import React, { useCallback, useState } from 'react';
import { AtIcon, AtList } from 'taro-ui';
import styles from './fileExplorer.module.scss';
import { previewFile } from '@root/utils/autoGCFileSystem';


function matchFileImage(extension) {
  switch (extension) {
    case 'ppt':
    case 'pptx':
      return 'PPT';
    case 'xls':
    case 'xlsx':
      return 'excel';
    case 'doc':
    case 'docx':
      return 'word';
    default:
      return extension;
  }
}

export default function FileExplorer(props) {
  const { className, node, nodeHash, gotoNode, keywords } = props;
  return (
    <ScrollView scrollY className={className}>
      <AtList className={styles.list}>
        {
          node && node.mapChildrenIds(nodeId => {
            const fileNode = nodeHash[nodeId];
            if (!fileNode) return null;
            if(keywords && fileNode.label.toLowerCase().indexOf(keywords) === -1) { return null; }
            return (
              fileNode.type === 'folder'
                ? (
                  <ListItem
                    key={fileNode.id}
                    className={styles.listItem}
                    title={fileNode.label}
                    arrow="right"
                    // note="xxx个项目"
                    onClick={() => gotoNode(fileNode)}
                    thumb={getFileUrl('oss://assets/file-icon/folder.png')}
                  />
                )
                : (
                  <ListItem
                    key={fileNode.id}
                    className={[styles.listItem, styles.fileItem]}
                    title={fileNode.label}
                    note={`版本: ${fileNode.data.version}`}
                    thumb={getFileUrl(`oss://assets/file-icon/${matchFileImage(fileNode.data.extension)}.svg`)}
                    extraThumb={
                      getFileUrl('oss://assets/file-icon/download.png')
                    }
                    ext={fileNode.data.extension}
                    onExtraThumbClick={() => {
                      previewFile(fileNode.data.ipfsCid, fileNode.label, fileNode.data.extension, fileNode.data.fileSize);
                    }}
                  />
                )
            )
          })
        }
      </AtList>
    </ScrollView>
  )
}


const deleteBtnWidth = 64;
function ListItem(props) {
  const { className, enableDelete = false, title, ext, note, arrow, thumb: propsThumb, defaultThumb = getFileUrl(`oss://assets/file-icon/default.svg`), extraThumb, onClick, onExtraThumbClick } = props;
  const [initX, setInitX] = useState();
  const [swipeWidth, setSwipeWidth] = useState(0);
  const [thumb, setThumb] = useState(propsThumb);
  const [showExt, setShowExt] = useState(false);

  const onTouchStart = useCallback((e) => {
    setInitX(e.touches[0].pageX);
  }, [enableDelete]);

  const onTouchMove = useCallback((e) => {
    const currentX = e.touches[0].pageX;
    const diffx = currentX - initX;
    setSwipeWidth(prevValue => {
      return diffx >= 0
        ? prevValue ? Math.max(deleteBtnWidth - diffx, 0) : 0 // 右滑
        : prevValue !== deleteBtnWidth ? Math.min(deleteBtnWidth, Math.abs(diffx)) : deleteBtnWidth; // 左滑
    });

  }, [initX, enableDelete]);

  const onTouchEnd = useCallback(() => {
    setSwipeWidth(prevWidth => prevWidth > deleteBtnWidth / 2 ? deleteBtnWidth : 0);
    setInitX();
  }, [enableDelete])


  return (
    <View
      className={[styles.listItem2, className]}
    >
      <View
        className={[styles.content, initX ? styles.dragging : '']}
        onTouchStart={enableDelete ? onTouchStart : undefined}
        onTouchMove={enableDelete ? onTouchMove : undefined}
        onTouchEnd={enableDelete ? onTouchEnd : undefined}
        style={{ right: `${swipeWidth * 2}rpx` }}
        onClick={onClick}
      >
        {
          thumb ? (
            <View className={styles.itemThumb}>
              <Image className={styles.img} src={thumb} onError={(...args) => {
                setThumb(defaultThumb);
                setShowExt(true);
              }} />
              {
                showExt
                  ? <Text className={styles.ext}>{ext}</Text>
                  : null
              }
            </View>
          )
            : null
        }
        <View className={styles.itemContent}>
          <View className={styles.itemContent_info}>
            <View className={styles.itemContent_info_title}>{title}</View>
            {
              note
                ? (
                  <View className={styles.itemContent_info_note}>{note}</View>
                )
                : null
            }
          </View>
        </View>

        <View className={styles.itemExtra}>
          {
            extraThumb
              ? (
                <View className={styles.itemExtraThumb} onClick={onExtraThumbClick}>
                  <Image className={styles.img} src={extraThumb} />
                </View>
              )
              : null
          }
          {
            arrow
              ? (
                <AtIcon className={styles.itemExtraIcon} value='chevron-right' />
              )
              : null
          }
        </View>
      </View>
      <View className={styles.actions}>
        <View
          className={styles.deleteBtn}
          style={{ width: `${deleteBtnWidth * 2}rpx` }}
        >删除</View>
      </View>
    </View>
  )
}