import Taro, { getFileSystemManager, hideLoading, showLoading } from "@tarojs/taro";
import { downloadFile } from "./request";
import { hint } from "./tool";

const singleFileMaxSize = 50 * 1024 * 1024;
const maxSpaceSize = 200 * 1024 * 1024;
const dir = wx.env.USER_DATA_PATH;
const genFilePath = (ipfsCid, extension, fileName) => `${dir}/${ipfsCid}/${fileName}.${extension}`;
const genFileFolder = ipfsCid => `${dir}/${ipfsCid}`;
const fileManager = getFileSystemManager();
const isAndroid = wx.getSystemInfoSync().platform === 'android';
const iosSupportFilesType = ['doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx', 'pdf'];
const oneOfType = ext => iosSupportFilesType.indexOf(ext) !== -1;


function openFile(filePath, extension, onFail) {
  Taro.openDocument({
    filePath: filePath,
    fileType: extension,
    showMenu: true,
    fail: (...args) => {
      console.log('open document failed:', args);
      onFail && onFail(...args);
    }
  })
}


function gcDirSpace() {
  const list = fileManager.statSync(dir, true).filter(fileStats => fileStats.stats.isFile());
  list.sort((fileA, fileB) => fileA.stats.lastAccessedTime > fileB.stats.lastAccessedTime ? 1 : -1);
  const dirSize = list.reduce((sum, { stats }) => sum + stats.size, 0);
  const remainningSpace = maxSpaceSize - dirSize;
  console.log('trigger file system gc before:', remainningSpace, list);
  if (remainningSpace > singleFileMaxSize) { return; }
  let cutSize = singleFileMaxSize - remainningSpace;
  console.log('trigger file system gc:', list);
  list.some(fileStat => {
    try {
      fileManager.unlinkSync(`${dir}${fileStat.path}`);
      cutSize = cutSize - fileStat.stats.size;
      if(cutSize < 0) return true;
      return false;
    } catch {
      return false;
    }
  });
  console.log('after file system gc:', fileManager.statSync(dir, true).filter(fileStats => fileStats.stats.isFile()));
}

export function previewFile(ipfsCid, fileName, extension, fileSize) {
  if (!isAndroid && !oneOfType(extension)) {
    hint('小程序目前仅支持ppt, excel, word, pdf文件的预览');
    return;
  }
  if (fileSize > singleFileMaxSize) {
    hint('文件过大，请通过pc端查看文件。');
    return;
  }
  const filePath = genFilePath(ipfsCid, extension, fileName);
  fileManager.access({
    path: filePath,
    success: () => {
      openFile(filePath, extension);
    },
    fail: () => {
      gcDirSpace();
      showLoading({ mask: true });
      fileManager.mkdirSync(genFileFolder(ipfsCid));
      const task = downloadFile(ipfsCid, {
        filePath,
        success: (param) => {
          hideLoading();
          openFile(param.filePath, extension);
        },
        fail: (e) => {
          hideLoading();
          console.log('download fail:', e);
          if (e.errMsg.indexOf('file data is empty') !== -1) {
            hint('文件内容为空');
            return;
          }
          hint(e.errMsg);
        }
      });
      const progressCb = task.progress || task.onProgressUpdate;
      if (progressCb) {
        progressCb(({ progress }) => {
          showLoading({ title: `${progress}%`, mask: true });
        })
      }
    }
  })
}
